# !/usr/bin/env python

# Imports
from pyniryo2 import *
import time
from threading import Event

# -- MUST Change these variables
gripper_used = ToolID.GRIPPER_2  # Tool used for picking
gpio_sensor = PinID.GPIO_1A  # Pin of the sensor

# Set robot address
robot_ip_address = "192.168.1.124"

# Events
calibrated_event = Event()
calibrated_event.clear()

update_tool_event = Event()
update_tool_event.clear

close_gripper_event = Event()
close_gripper_event.clear

# -- Should Change these variables

# Poses
conveyor_pose_2 = PoseObject(
    x=0.193, y=0.199, z=0.19,
    roll=-0.071, pitch=1.527, yaw=-0.090)

conveyor_pose_1 = PoseObject(
    x=0.0, y=0.2, z=0.19,
    roll=-0.3, pitch=1.47, yaw=-0.079)

storage_pose = PoseObject(
    x=0.25, y=-0.1, z=0.135,
    roll=-0.77, pitch=1.57, yaw=-2.2)

# Variables
offset = 0.06

nb_el = 3

# Callbacks
def update_tool_success_callback(result):
    time.sleep(2)
    update_tool_event.set()
    print 'Update Tool: ', result['message']

def update_tool_error_callback(result):
    print 'Update Tool: ', result['message']

def calibrate_success_callback(result):
    calibrated_event.set()
    print 'Calibrate Callback: ', result["message"]

def calibrate_error_callback(result):
    print 'Calibrate Callback: ', result["message"]

def close_gripper_callback(result):
    print 'Close Callback: ', result["message"]

# Pick and place with conveyor
def pick_n_place_w_conveyor(robot):
    """
    Simple pick and place with a conveyor belt

    :type niyro_robot: NiryoRobot
    :rtype: None
    """
    # Run conveyor
    robot.conveyor.run_conveyor(conveyor_id)

    # Wait for sensor to turn to low state which means it has something in front of it
    while not robot.io.digital_read(gpio_sensor) == PinState.LOW:
        print(robot.io.digital_read(gpio_sensor))
    
    # Stop Conveyor
    robot.conveyor.stop_conveyor(conveyor_id)

    # Pick and place to storage position
    robot.pick_place.pick_and_place(conveyor_pose_2, storage_pose)

if __name__ == '__main__':
    # Connect to robot
    robot = NiryoRobot(robot_ip_address)

    # Set Pin Mode
    robot.io.set_pin_mode(gpio_sensor, PinMode.INPUT)
    
    # Enable connection with conveyor
    conveyor_id = robot.conveyor.set_conveyor()
    print 'Conveyor ID: ', conveyor_id

    # Calibrate robot if robot needs calibration
    robot.arm.calibrate_auto(callback=calibrate_success_callback, errback=calibrate_error_callback)
    calibrated_event.wait(20)
    if not calibrated_event.is_set():
        quit

    # Update tool
    robot.tool.update_tool(callback=update_tool_success_callback, errback=update_tool_error_callback)
    update_tool_event.wait()

    # Launching main process
    for i in range(nb_el):
        robot.pick_place.pick_and_place(storage_pose, conveyor_pose_1)
        if i < 2:
            storage_pose.y += offset
            conveyor_pose_1.x -= offset
    
    for i in range(nb_el):
        pick_n_place_w_conveyor(robot)
        storage_pose.y -= offset

    robot.tool.close_gripper(callback=close_gripper_callback)

    # Releasing connection
    robot.arm.go_to_sleep()
    robot.end()



#!/usr/bin/env python
import unittest
from threading import Event
import numpy as np
import time

from pyniryo2.exceptions import RobotCommandException
from pyniryo2.niryo_ros import NiryoRos
from pyniryo2.objects import PoseObject

from pyniryo2.trajectories.trajectories import Trajectories
from pyniryo2.arm.arm import Arm
from pyniryo2.objects import PoseObject
from trajectory_msgs.msg import JointTrajectory, JointTrajectoryPoint
from std_msgs.msg import Int32, Bool, Header

robot_ip_address = "127.0.0.1"
port = 9090

test_order = ["test_creation_delete_trajectory",
              "test_last_learned_trajectory",
              "test_execute_trajectory",
              "test_execute_trajectory_type",
              "test_execute_registered_trajectory"
              ]


class BaseTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.client = NiryoRos(ip_address=robot_ip_address, port=port)
        cls.trajectories = Trajectories(cls.client)
        cls.arm = Arm(cls.client)

    @classmethod
    def tearDownClass(cls):
        pass
        # cls.arm.go_to_sleep()
        # cls.client.close()

    @staticmethod
    def assertAlmostEqualVector(a, b, decimal=1):
        np.testing.assert_almost_equal(a, b, decimal)

    def assertEqualUnsortedList(self, a, b):
        a_sorted = a[:]
        a_sorted.sort()
        b_sorted = b[:]
        b_sorted.sort()
        self.assertEqual(a_sorted, b_sorted)


# noinspection PyTypeChecker
class TestTrajectories(BaseTest):
    trajectory = [[-0.493, -0.32, -0.505, -0.814, -0.282, 0],
                  [0.834, -0.319, -0.466, 0.822, -0.275, 0],
                  [1.037, -0.081, 0.248, 1.259, -0.276, 0]]

    robot_poses = [[0.3, 0.1, 0.3, 0., 0., 0., 1.],
                   [0.3, -0.1, 0.3, 0., 0., 0., 1.],
                   [0.3, -0.1, 0.4, 0., 0., 0., 1.],
                   [0.3, 0.1, 0.4, 0., 0., 0., 1.]]

    neutral_pose = [0.288, -0.015, 0.441, 0.035, -0.05, -0.05]

    def go_to_neutral_pose(self):
        self.assertIsNone(self.arm.move_pose(self.neutral_pose))
        self.assertAlmostEqualVector(self.arm.get_pose().to_list(), self.neutral_pose)

    def test_creation_delete_trajectory(self):
        self.assertIsNone(self.trajectories.clean_trajectory_memory())
        # Get saved trajectory list & copy it
        result = self.trajectories.get_saved_trajectory_list()
        self.assertIsInstance(result, list)
        traj_list = self.trajectories.get_saved_trajectory_list()

        # Create new trajectories
        for i in range(3):
            new_name = 'unittest_{:03d}'.format(i)
            new_description = 'unittest_description_{:03d}'.format(i)
            self.assertIsNone(self.trajectories.save_trajectory(self.trajectory, new_name, new_description))
            self.assertEqualUnsortedList(self.trajectories.get_saved_trajectory(new_name), self.trajectory)
            if new_name not in [traj[0] for traj in traj_list]:
                traj_list.append((new_name, new_description))

            result = self.trajectories.get_saved_trajectory_list()
            self.assertEqualUnsortedList(result, traj_list)

        # Update trajectories
        old_names = list([traj[0] for traj in traj_list])
        for i in range(3):
            old_name = 'unittest_{:03d}'.format(i)
            new_name = 'unittest_update_{:03d}'.format(i)
            new_description = 'unittest_update_description_{:03d}'.format(i)
            self.assertIsNone(self.trajectories.update_trajectory_infos(old_name, new_name, new_description))
            self.assertEqualUnsortedList(self.trajectories.get_saved_trajectory(new_name), self.trajectory)
            try:
                name_index = old_names.index(old_name)
                traj_list[name_index] = (new_name, new_description)
            except ValueError:
                pass

            result = self.trajectories.get_saved_trajectory_list()
            self.assertEqualUnsortedList(result, traj_list)

        # Delete created trajectories
        old_names = list([traj[0] for traj in traj_list])
        for i in range(3):
            traj_name = 'unittest_update_{:03d}'.format(i)
            old_names.remove(traj_name)

            self.assertIsNone(self.trajectories.delete_trajectory(traj_name))
            time.sleep(1.0)
            result = self.trajectories.get_saved_trajectory_list()
            self.assertListEqual([traj[0] for traj in result], old_names)

            with self.assertRaises(RobotCommandException):
                self.trajectories.delete_trajectory(traj_name)

    def test_last_learned_trajectory(self):
        for i in range(3):
            new_name = 'unittest_{:03d}'.format(i)
            new_description = 'unittest_description_{:03d}'.format(i)
            self.assertIsNone(self.trajectories.save_trajectory(self.trajectory, new_name, new_description))

        self.assertIsNone(self.trajectories.clean_trajectory_memory())
        self.assertEqual(self.trajectories.get_saved_trajectory_list(), [])

        self.assertIsNone(self.trajectories.save_trajectory(self.trajectory, "last_executed_trajectory", ""))
        self.assertIsNone(self.trajectories.save_last_learned_trajectory("unittest_name", "unittest_description"))

        result = self.trajectories.get_saved_trajectory_list()
        self.assertEqual(result, [("unittest_name", "unittest_description")])
        self.assertIsNone(self.trajectories.clean_trajectory_memory())
        self.assertEqual(self.trajectories.get_saved_trajectory_list(), [])

    def test_execute_trajectory(self):
        # Testing trajectory from poses
        self.assertIsNone(self.arm.calibrate_auto())
        self.go_to_neutral_pose()
        self.assertIsNone(self.trajectories.execute_trajectory_from_poses(self.robot_poses))
        self.assertAlmostEqualVector(self.arm.get_pose().quaternion_pose, self.robot_poses[-1])

        trajectory_event = Event()
        trajectory_event.clear()

        def trajectory_callback(_):
            trajectory_event.set()

        self.go_to_neutral_pose()
        self.assertIsNone(
            self.trajectories.execute_trajectory_from_poses(self.robot_poses, dist_smoothing=0.05,
                                                            callback=trajectory_callback))
        self.assertTrue(trajectory_event.wait(20))
        self.assertAlmostEqualVector(self.arm.get_pose().quaternion_pose, self.robot_poses[-1])

    def test_execute_trajectory_type(self):
        self.assertIsNone(self.arm.calibrate_auto())
        self.go_to_neutral_pose()
        self.trajectories.execute_trajectory_from_poses([[0.3, 0.1, 0.3, 0., 0., 0., 1.],
                                                         PoseObject(0.3, -0.1, 0.3, 0., 0., 0.),
                                                         [0.3, -0.1, 0.4, 0., 0., 0.],
                                                         PoseObject(0.3, 0.1, 0.4, 0., 0., 0.)])
        self.assertAlmostEqualVector(self.arm.get_pose().quaternion_pose, self.robot_poses[-1])

        with self.assertRaises(RobotCommandException):
            self.trajectories.execute_trajectory_from_poses(0)

        with self.assertRaises(RobotCommandException):
            self.trajectories.execute_trajectory_from_poses(PoseObject(0, 0, 0, 0, 0, 0))

        with self.assertRaises(RobotCommandException):
            self.trajectories.execute_trajectory_from_poses([0.3, 0.1, 0.3, 0., 0., 0., 1.])

        with self.assertRaises(RobotCommandException):
            self.trajectories.execute_trajectory_from_poses([[0.3, 0.1, 0.3, 0., 0.]])

        with self.assertRaises(RobotCommandException):
            self.trajectories.execute_trajectory_from_poses(self.robot_poses, dist_smoothing=-0.1)

    def test_execute_registered_trajectory(self):
        # Testing trajectory from poses
        self.assertIsNone(self.arm.calibrate_auto())

        trajectory_event = Event()
        trajectory_event.clear()

        def trajectory_callback(_):
            trajectory_event.set()

        # Create & save a trajectory, then execute it & eventually delete it
        traj_name = "test_trajectory_save_and_execute_registered"
        traj_description = "test_trajectory_save_and_execute_registered_description"
        # self.go_to_neutral_pose()
        self.assertIsNone(self.trajectories.save_trajectory(self.trajectory, traj_name, traj_description))
        self.assertIsNone(self.trajectories.execute_registered_trajectory(traj_name, callback=trajectory_callback))

        trajectory_event.clear()
        self.assertIsNone(self.arm.move_pose(self.neutral_pose))
        self.assertIsNone(self.trajectories.delete_trajectory(traj_name))

        with self.assertRaises(RobotCommandException):
            self.assertIsNone(self.trajectories.execute_registered_trajectory(traj_name))


def suite():
    suite = unittest.TestSuite()
    for function_name in test_order:
        suite.addTest(TestTrajectories(function_name))
    return suite


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())



import roslibpy

from pyniryo2.utils import pose_list_to_dict, point_list_to_dict

from .enums import ManageWorkspace


class VisionServices(object):

    def __init__(self, client):
        self.__client = client

        self.get_target_pose_service = roslibpy.Service(self.__client,
                                                        '/niryo_robot_poses_handlers/get_target_pose',
                                                        'niryo_robot_vision/GetTargetPose')

        self.obj_detection_rel_service = roslibpy.Service(self.__client,
                                                          '/niryo_robot_vision/obj_detection_rel',
                                                          'niryo_robot_vision/ObjDetection')

        self.manage_workspace_service = roslibpy.Service(self.__client,
                                                         '/niryo_robot_poses_handlers/manage_workspace',
                                                         'niryo_robot_poses_handlers/ManageWorkspace')

        self.get_workspace_poses_service = roslibpy.Service(self.__client,
                                                            '/niryo_robot_poses_handlers/get_workspace_poses',
                                                            'niryo_robot_poses_handlers/GetWorkspaceRobotPoses')

        self.get_workspace_ratio_service = roslibpy.Service(self.__client,
                                                            '/niryo_robot_poses_handlers/get_workspace_ratio',
                                                            'niryo_robot_msgs/GetWorkspaceRatio')

        self.get_workspace_list_service = roslibpy.Service(self.__client,
                                                           '/niryo_robot_poses_handlers/get_workspace_list',
                                                           'niryo_robot_msgs/GetNameDescriptionList')

        self.set_brightness_service = roslibpy.Service(self.__client,
                                                       '/niryo_robot_vision/set_brightness',
                                                       'niryo_robot_vision/SetImageParameter')

        self.set_contrast_service = roslibpy.Service(self.__client,
                                                     '/niryo_robot_vision/set_contrast',
                                                     'niryo_robot_vision/SetImageParameter')

        self.set_saturation_service = roslibpy.Service(self.__client,
                                                       '/niryo_robot_vision/set_saturation',
                                                       'niryo_robot_vision/SetImageParameter')

    @staticmethod
    def get_trigger_request():
        return roslibpy.ServiceRequest()

    @staticmethod
    def get_target_pose_service_request(workspace, height_offset, x_rel, y_rel, yaw_rel):
        return roslibpy.ServiceRequest(
            {"workspace": workspace, "height_offset": height_offset, "x_rel": x_rel, "y_rel": y_rel,
             "yaw_rel": yaw_rel})

    @staticmethod
    def obj_detection_rel_service_request(obj_type, obj_color, workspace_ratio, ret_image=False):
        return roslibpy.ServiceRequest(
            {"obj_type": obj_type.value, "obj_color": obj_color.value, "workspace_ratio": workspace_ratio,
             "ret_image": ret_image})

    @staticmethod
    def manage_workspace_service_request(cmd, workspace):
        if not isinstance(cmd, ManageWorkspace):
            raise TypeError
        return roslibpy.ServiceRequest({"cmd": cmd.value, "workspace": workspace})

    @staticmethod
    def add_workspace_from_poses_request(workspace_name, pose_list):
        workspace = {"name": workspace_name, "poses": [pose_list_to_dict(pose) for pose in pose_list]}
        return VisionServices.manage_workspace_service_request(ManageWorkspace.SAVE, workspace)

    @staticmethod
    def add_workspace_from_points_request(workspace_name, point_list):
        workspace = {"name": workspace_name,
                     "points": [point_list_to_dict(point) for point in point_list]}
        return VisionServices.manage_workspace_service_request(ManageWorkspace.SAVE_WITH_POINTS, workspace)

    @staticmethod
    def get_workspace_poses_service_request(workspace_name):
        return roslibpy.ServiceRequest({"name": workspace_name})

    @staticmethod
    def get_workspace_ratio_service_request(workspace_name):
        return roslibpy.ServiceRequest({"workspace": workspace_name})

    @staticmethod
    def get_workspace_list_service_request():
        return VisionServices.get_trigger_request()

    @staticmethod
    def get_image_parameter_request(factor):
        return roslibpy.ServiceRequest({"factor": factor})



import roslibpy

from pyniryo2.io.enums import PinID, PinState


class IOServices(object):

    def __init__(self, client):
        self.__client = client

        self.set_digital_io_mode_service = roslibpy.Service(self.__client,
                                                            '/niryo_robot_rpi/set_digital_io_mode',
                                                            'niryo_robot_rpi/SetIOMode')

        self.set_digital_io_state_service = roslibpy.Service(self.__client,
                                                             '/niryo_robot_rpi/set_digital_io',
                                                             'niryo_robot_rpi/SetDigitalIO')

        self.get_digital_io_service = roslibpy.Service(self.__client,
                                                       '/niryo_robot_rpi/get_digital_io',
                                                       'niryo_robot_rpi/GetDigitalIO')

        self.set_analog_io_state_service = roslibpy.Service(self.__client,
                                                            '/niryo_robot_rpi/set_analog_io',
                                                            'niryo_robot_rpi/SetAnalogIO')

        self.get_analog_io_service = roslibpy.Service(self.__client,
                                                      '/niryo_robot_rpi/get_analog_io',
                                                      'niryo_robot_rpi/GetAnalogIO')

    @staticmethod
    def set_io_mode_request(pin, value):
        pin_name = pin.value if isinstance(pin, PinID) else pin
        return roslibpy.ServiceRequest({"name": pin_name, "mode": value.value})

    @staticmethod
    def set_io_state_request(pin, value):
        pin_name = pin.value if isinstance(pin, PinID) else pin
        pin_value = value.value if isinstance(value, PinState) else value
        return roslibpy.ServiceRequest({"name": pin_name, "value": pin_value})

    @staticmethod
    def get_io_request(pin):
        pin_name = pin.value if isinstance(pin, PinID) else pin
        return roslibpy.ServiceRequest({"name": pin_name})



from enum import Enum, unique


@unique
class AnimationMode(Enum):
    """
    Enumeration of animations available for the user to control the Led Ring
    """
    NONE = -1
    UNKNOWN = 0
    SOLID = 1
    FLASHING = 2
    ALTERNATE = 3
    CHASE = 4
    COLOR_WIPE = 5
    RAINBOW = 6
    RAINBOW_CYLE = 7
    RAINBOW_CHASE = 8
    GO_UP = 9
    GO_UP_AND_DOWN = 10
    BREATH = 11
    SNAKE = 12
    CUSTOM = 13

class LedMode(Enum):
    """
    Enumeration of available Led Mode
    """
    NONE = -1
    UNKNOWN = 0
    ROBOT_STATUS = 1
    USER = 2



from pyniryo2.io.enums import PinMode
from pyniryo2.io.objects import DigitalPinObject, AnalogPinObject
from pyniryo2.niryo_topic import NiryoTopic


class IOTopics(object):

    def __init__(self, client):
        self.__client = client

        self.digital_io_topic = NiryoTopic(self.__client,
                                           '/niryo_robot_rpi/digital_io_state',
                                           'niryo_robot_rpi/DigitalIOState',
                                           digital_io_state_topic_conversion)

        self.analog_io_topic = NiryoTopic(self.__client,
                                          '/niryo_robot_rpi/analog_io_state',
                                          'niryo_robot_rpi/AnalogIOState',
                                          analog_io_state_topic_conversion)

        self.__custom_button_topic = NiryoTopic(self.__client,
                                                '/niryo_robot_hardware_interface/end_effector_interface/custom_button_status',
                                                'end_effector_interface/EEButtonStatus',
                                                custom_button_topic_conversion)


def digital_io_state_topic_conversion(msg):
    digital_inputs = [DigitalPinObject(str(di['name']), PinMode.INPUT, di['value']) for di in msg["digital_inputs"]]
    digital_outputs = [DigitalPinObject(str(do['name']), PinMode.OUTPUT, do['value']) for do in msg["digital_outputs"]]
    return digital_inputs + digital_outputs


def analog_io_state_topic_conversion(msg):
    analog_inputs = [AnalogPinObject(str(ai['name']), PinMode.INPUT, ai['value']) for ai in msg["analog_inputs"]]
    analog_outputs = [AnalogPinObject(str(ao['name']), PinMode.OUTPUT, ao['value']) for ao in msg["analog_outputs"]]
    return analog_inputs + analog_outputs


def custom_button_topic_conversion(msg):
    return msg["action"] != msg["NO_ACTION"]



from enum import Enum, unique


@unique
class ConveyorID(Enum):
    """
    ConveyorID to be able to have CAN (id 12 and 13) and TTL (id 9 and 10) conveyor in any possible combination

    ID_1 = 12   # One, Ned
    ID_2 = 13   # One, Ned
    ID_3 = 9    # Ned2
    ID_4 = 10   # Ned2
    """
    NONE = 0
    ID_1 = -1
    ID_2 = -2


@unique
class ConveyorCan(Enum):
    """
    ConveyorID to control conveyors with CAN interface
    """

    NONE = 0
    ID_1 = 12
    ID_2 = 13


@unique
class ConveyorTTL(Enum):
    """
    ConveyorID to control conveyors with TTL interface
    """

    NONE = 0
    ID_1 = 9
    ID_2 = 10


@unique
class ConveyorDirection(Enum):
    """
    Enumeration of the directions of the conveyor
    """

    FORWARD = 1
    BACKWARD = -1


@unique
class ConveyorStatus(Enum):
    """
    Enumeration of the different Conveyor status
    """

    ADD = 1
    REMOVE = 2



# coding=utf-8

from io import open
from setuptools import find_packages, setup

version = '1.0.1'


with open('README.md', 'r', encoding='utf-8') as f:
    readme = f.read()

with open('requirements.txt', 'r') as f:
    REQUIRES = [str(line.replace("\n", "")) for line in f.readlines()]

kwargs = {
    'name': 'pyniryo2',
    'version': version,
    'description': 'Package to control Niryo Robot "Ned" through TCP',
    'long_description': readme,
    'long_description_content_type': 'text/markdown',
    'author': 'Niryo',
    'author_email': 'v.pitre@niryo.com',
    'maintainer': 'Valentin Pitre',
    'maintainer_email': 'v.pitre@niryo.com',
    'install_requires': REQUIRES,
    'include_package_data': True,
    'url': 'https://github.com/NiryoRobotics/pyniryo2',
    'license': 'GNU 3.0',
    'classifiers': [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Topic :: Education",
    ],
    'packages': find_packages(exclude=('tests', 'tests.*')),
}

setup(**kwargs)



#!/usr/bin/env python
# coding=utf-8

from collections import namedtuple

CameraInfo = namedtuple("CameraInfo", ['intrinsics', 'distortion'])

ImageParameters = namedtuple("ImageParameters", ['brightness_factor', 'contrast_factor', 'saturation_factor'])



import roslibpy

from pyniryo2.utils import pose_quat_dict_to_list

from .enums import ManageTrajectories


class TrajectoriesServices(object):

    def __init__(self, client):
        self.__client = client

        self.get_trajectory_from_name_service = roslibpy.Service(self.__client,
                                                                 '/niryo_robot_arm_commander/get_trajectory',
                                                                 'niryo_robot_arm_commander/GetTrajectory')

        self.trajectory_manager_service = roslibpy.Service(self.__client,
                                                           '/niryo_robot_arm_commander/manage_trajectory',
                                                           'niryo_robot_arm_commander/ManageTrajectory')

        self.get_trajectory_list_service = roslibpy.Service(self.__client,
                                                            '/niryo_robot_arm_commander/get_trajectory_list',
                                                            'niryo_robot_arm_commander/GetNameDescriptionList')

    @staticmethod
    def get_trajectory_from_name_request(traj_name):
        return roslibpy.ServiceRequest({"name": traj_name})

    @staticmethod
    def save_last_learned_trajectory_request(name, description=""):
        return roslibpy.ServiceRequest(
            {"cmd": ManageTrajectories.SAVE_LAST_LEARNED.value, "name": name, "description": description})

    @staticmethod
    def save_trajectory_request(trajectory, name, description=""):
        return roslibpy.ServiceRequest(
            {"cmd": ManageTrajectories.SAVE.value, "trajectory": trajectory, "name": name, "description": description})

    @staticmethod
    def delete_trajectory_request(name):
        return roslibpy.ServiceRequest({"cmd": ManageTrajectories.DELETE.value, "name": name})

    @staticmethod
    def clean_trajectory_memory_request():
        return roslibpy.ServiceRequest({"cmd": ManageTrajectories.DELETE_ALL.value})

    @staticmethod
    def execute_registered_trajectory_request(name):
        return roslibpy.ServiceRequest({"cmd": ManageTrajectories.EXECUTE_REGISTERED.value, "name": name})

    @staticmethod
    def update_trajectory_infos_request(name, new_name, description=""):
        return roslibpy.ServiceRequest(
            {"cmd": ManageTrajectories.UPDATE.value, "name": name, "new_name": new_name, "description": description})

    @staticmethod
    def get_saved_trajectory_list_request():
        return roslibpy.ServiceRequest()

    @staticmethod
    def pose_trajectory_dict_to_list(traj_dict):
        return [pose_quat_dict_to_list(pose_dict) for pose_dict in traj_dict]

    @staticmethod
    def trajectory_to_dict(trajectory):
        traj_dict = {
            'joint_names': ["joint_1", "joint_2", "joint_3", "joint_4", "joint_5", "joint_6"],
            'points': [{'positions': positions} for positions in trajectory]
        }
        return traj_dict

    @staticmethod
    def trajectory_dict_to_list(traj_dict):
        return [point["positions"] for point in traj_dict["points"]]



#!/usr/bin/env python
# coding=utf-8

from pyniryo2.led_ring.enums import AnimationMode, LedMode


class LedRingStatusObject:
    """
    Object used to store Led Ring status
    """

    def __init__(self):
        self.r = 0
        self.g = 0
        self.b = 0

        self.animation = AnimationMode.UNKNOWN
        self.mode = LedMode.UNKNOWN

    def init_from_message(self, msg):
        self.r = msg['animation_color']['r']
        self.g = msg['animation_color']['g']
        self.b = msg['animation_color']['b']

        self.animation = AnimationMode(msg['animation_mode']['animation'])
        self.mode = LedMode(msg['led_mode'])



"""
This script shows an example of how to use Ned's vision to
make a conditioning according to the objects' color.

The objects will be conditioned in a grid of dimension grid_dimension. The Y axis corresponds
to the ObjectColor : BLUE / RED / GREEN. It will be 3
The X axis corresponds to how many objects can be put on the same line before increasing
the conditioning height.
Once a line is completed, objects will be pack over the lower level
"""

from pyniryo2 import *

# -- MUST Change these variables
simulation_mode = True
if simulation_mode:
    robot_ip_address, workspace_name = "127.0.0.1", "gazebo_1"
else:
    robot_ip_address, workspace_name = "10.10.10.10", "workspace_1"

# -- Can change these variables
grid_dimension = (3, 3)

# -- Should Change these variables
# The pose from where the image processing happen
observation_pose = PoseObject(
    x=0.20, y=0., z=0.3,
    roll=0.0, pitch=1.57, yaw=0.0,
)

# Center of the conditioning area
first_conditioning_pose = PoseObject(
    x=0.0, y=-0.25, z=0.12,
    roll=-0., pitch=1.57, yaw=-1.57
)


# -- MAIN PROGRAM
def process(niryo_robot):
    """

    :type niryo_robot: NiryoRobot
    :rtype: None
    """
    try_without_success = 0
    count_dict = {
        ObjectColor.BLUE: 0,
        ObjectColor.RED: 0,
        ObjectColor.GREEN: 0,
    }
    # Loop until too much failures
    while try_without_success < 3:
        # Moving to observation pose
        niryo_robot.arm.move_pose(observation_pose)
        # Trying to get object via Vision Pick
        obj_found, shape, color = niryo_robot.vision.vision_pick(workspace_name)
        if not obj_found:
            try_without_success += 1
            continue

        # Choose X & Z position according to how the color line is filled
        offset_x_ind = count_dict[color] % grid_dimension[0]
        offset_z_ind = count_dict[color] // grid_dimension[0]

        # Choose Y position according to ObjectColor
        if color == ObjectColor.BLUE:
            offset_y_ind = -1
        elif color == ObjectColor.RED:
            offset_y_ind = 0
        else:
            offset_y_ind = 1
        # Going to place the object
        place_pose = first_conditioning_pose.copy_with_offsets(0.05 * offset_x_ind,
                                                               0.05 * offset_y_ind,
                                                               0.025 * offset_z_ind)
        niryo_robot.pick_place.place_from_pose(place_pose)
        # Increment count
        count_dict[color] += 1
        try_without_success = 0


if __name__ == '__main__':
    # Connect to robot
    robot = NiryoRobot(robot_ip_address)
    # Changing tool
    robot.tool.update_tool()
    # Calibrate robot if robot needs calibration
    robot.arm.calibrate_auto()
    # Launching main process
    process(robot)
    # Ending
    robot.arm.go_to_sleep()
    # Releasing connection
    robot.end()



from enum import Enum, unique

@unique
class RobotErrors(Enum):
    
    # overall behavior
    SUCCESS = 1
    CANCELLED = 2
    PREEMPTED = 3

    FAILURE = -1
    ABORTED = -3
    STOPPED = -4

    ROS_ERROR = -20

    FILE_ALREADY_EXISTS = -30

    UNKNOWN_COMMAND = -50
    NOT_IMPLEMENTED_COMMAND = -51
    INVALID_PARAMETERS = -52

    # - Hardware
    HARDWARE_FAILURE = -110
    HARDWARE_NOT_OK = -111
    LEARNING_MODE_ON = -112
    CALIBRATION_NOT_DONE = -113
    DIGITAL_IO_PANEL_ERROR = -114
    LED_MANAGER_ERROR = -115
    BUTTON_ERROR = -116
    WRONG_MOTOR_TYPE = -117
    DXL_WRITE_ERROR = -118
    DXL_READ_ERROR = -119
    CAN_WRITE_ERROR = -120
    CAN_READ_ERROR = -121
    NO_CONVEYOR_LEFT = -122
    NO_CONVEYOR_FOUND = -123
    CONVEYOR_ID_INVALID = -124
    CALIBRATION_IN_PROGRESS = -125

    # - Vision
    VIDEO_STREAM_ON_OFF_FAILURE = -170
    VIDEO_STREAM_NOT_RUNNING = -171
    OBJECT_NOT_FOUND = -172
    MARKERS_NOT_FOUND = -173

    # - Commander
    # Arm Commander
    ARM_COMMANDER_FAILURE = -220
    GOAL_STILL_ACTIVE = -221
    JOG_CONTROLLER_ENABLED = -222
    CONTROLLER_PROBLEMS = -223
    SHOULD_RESTART = -224
    JOG_CONTROLLER_FAILURE = -225

    PLAN_FAILED = -230
    NO_PLAN_AVAILABLE = -231
    INVERT_KINEMATICS_FAILURE = -232

    # Tool Commander
    TOOL_FAILURE = -251
    TOOL_ID_INVALID = -252
    TOOL_NOT_CONNECTED = -253
    TOOL_ROS_INTERFACE_ERROR = -254

    # - Pose Handlers
    POSES_HANDLER_CREATION_FAILED = -300
    POSES_HANDLER_REMOVAL_FAILED = -301
    POSES_HANDLER_READ_FAILURE = -302
    POSES_HANDLER_COMPUTE_FAILURE = -303

    WORKSPACE_CREATION_FAILED = -308

    # - Programs Manager
    PROGRAMS_MANAGER_FAILURE = -320
    PROGRAMS_MANAGER_READ_FAILURE = -321
    PROGRAMS_MANAGER_UNKNOWN_LANGUAGE = -325
    PROGRAMS_MANAGER_NOT_RUNNABLE_LANGUAGE = -326
    PROGRAMS_MANAGER_EXECUTION_FAILED = -327
    PROGRAMS_MANAGER_STOPPING_FAILED = -328
    PROGRAMS_MANAGER_AUTORUN_FAILURE = -329
    PROGRAMS_MANAGER_WRITING_FAILURE = -330
    PROGRAMS_MANAGER_FILE_ALREADY_EXISTS = -331
    PROGRAMS_MANAGER_FILE_DOES_NOT_EXIST = -332


    # - Serial
    SERIAL_FILE_ERROR = -400
    SERIAL_UNKNOWN_ERROR = -401

    # - MQTT Client
    MQTT_PUBLISH_FUNCTION_DOESNT_EXIST = -420
    MQTT_PUBLISH_FUNCTION_INVALID_ARGUMENTS = -421

    # - System Api Client
    SYSTEM_API_CLIENT_UNKNOWN_ERROR = -440
    SYSTEM_API_CLIENT_INVALID_ROBOT_NAME = -441
    SYSTEM_API_CLIENT_REQUEST_FAILED = -442

@unique
class ArmMoveCommandType(Enum):
    """
    Enumeration of Arm Move Command : it used for move commands
    """
    JOINTS = 0            # uses joints 
    POSE = 1              # uses position and rpy 
    POSITION = 2          # uses position
    RPY = 3               # uses rpy
    POSE_QUAT = 4         # uses position and orientation
    LINEAR_POSE = 5       # uses position and rpy
    SHIFT_POSE = 6        # uses shift
    SHIFT_LINEAR_POSE = 7 # uses shift
    EXECUTE_TRAJ = 8      # uses dist_smoothing, list_poses
    DRAW_SPIRAL = 9
    DRAW_CIRCLE = 10
    EXECUTE_FULL_TRAJ = 11
    EXECUTE_RAW_TRAJ = 12



from enum import Enum, unique


@unique
class ManageSound(Enum):
    """
    Enumeration of the actions of sound database management
    """
    ADD = 1
    DELETE = 2


@unique
class Language(Enum):
    """
    Enumeration of the Text To Speech languages
    """
    ENGLISH = 0
    FRENCH = 1
    SPANISH = 3
    MANDARIN = 4
    PORTUGUESE = 5



from enum import Enum, unique

@unique
class CalibrateMode(Enum):
    """
    Enumeration of Calibration Modes
    """
    AUTO = 1
    MANUAL = 2


@unique
class RobotAxis(Enum):
    """
    Enumeration of Robot Axis : it used for Shift command
    """
    X = 0
    Y = 1
    Z = 2
    ROLL = 3
    PITCH = 4
    YAW = 5

@unique
class JogShift(Enum):
    """
    Enumeration of Jog Shift : it used for Jog commands
    """
    JOINTS_SHIFT = 1
    POSE_SHIFT = 2



#!/usr/bin/env python
import unittest
import numpy as np
from threading import Event

from pyniryo2.exceptions import RobotCommandException
from pyniryo2.niryo_topic import NiryoTopic
from pyniryo2.objects import PoseObject
from pyniryo2.niryo_ros import NiryoRos

from pyniryo2.vision.vision import Vision
from pyniryo2.vision.objects import CameraInfo, ImageParameters
from pyniryo2.vision.enums import ObjectColor

robot_ip_address = "127.0.0.1"
port = 9090

test_order = ["test_camera_info",
              "test_camera_img",
              "test_image_parameters",
              "test_workspace",
              "test_target_from_rel",
              ]


class BaseTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.client = NiryoRos(ip_address=robot_ip_address, port=port)
        cls.vision = Vision(cls.client)

    @classmethod
    def tearDownClass(cls):
        cls.client.close()

    @staticmethod
    def assertAlmostEqualVector(a, b, decimal=1):
        np.testing.assert_almost_equal(a, b, decimal)


# noinspection PyTypeChecker
class TestVision(BaseTest):

    def test_camera_info(self):
        self.assertIsInstance(self.vision.get_camera_intrinsics, NiryoTopic)
        self.assertIsInstance(self.vision.get_camera_intrinsics(), CameraInfo)

        cam_event = Event()
        cam_event.clear()

        def camera_info_callback(cam_info):
            self.assertIsInstance(cam_info, CameraInfo)
            cam_event.set()

        self.assertIsNone(self.vision.get_camera_intrinsics.subscribe(camera_info_callback))
        self.assertTrue(cam_event.wait(10))
        self.assertIsNone(self.vision.get_camera_intrinsics.unsubscribe())

    def test_camera_img(self):
        self.assertIsInstance(self.vision.get_img_compressed, NiryoTopic)
        self.assertIsInstance(self.vision.get_img_compressed(), bytes)

        cam_event = Event()
        cam_event.clear()

        def camera_img_callback(img):
            cam_event.set()
            self.assertIsInstance(img, bytes)

        self.assertIsNone(self.vision.get_img_compressed.subscribe(camera_img_callback))
        self.assertTrue(cam_event.wait(10))
        self.assertIsNone(self.vision.get_img_compressed.unsubscribe())

    def test_image_parameters(self):
        self.assertIsInstance(self.vision.get_image_parameters, NiryoTopic)
        self.assertIsInstance(self.vision.get_image_parameters(), ImageParameters)

        img_param_event = Event()
        img_param_event.clear()

        def img_param_callback(img_param):
            self.assertIsInstance(img_param, ImageParameters)
            img_param_event.set()

        self.assertIsNone(self.vision.get_image_parameters.subscribe(img_param_callback))
        self.assertTrue(img_param_event.wait(10))
        self.assertIsNone(self.vision.get_image_parameters.unsubscribe())

        old_img_param = self.vision.get_image_parameters.value
        for i, function in enumerate(
                [self.vision.set_brightness, self.vision.set_contrast, self.vision.set_saturation]):

            new_value = old_img_param[i] + 1.0
            self.assertIsNone(function(new_value))
            self.assertEqual(self.vision.get_image_parameters()[i], new_value)
            self.assertIsNone(function(1))
            self.assertEqual(self.vision.get_image_parameters()[i], 1)

            with self.assertRaises(RobotCommandException):
                function("1")

            with self.assertRaises(RobotCommandException):
                function(True)

    def test_workspace(self):
        ws_name = "unit_test_ws"
        self.assertIsNone(self.vision.delete_workspace(ws_name))

        unit_test_ws_poses = [ws_name,
                              [0.3, -0.1, 0.0, 0.0, 1.57, 0.0],
                              PoseObject(0.3, 0.1, 0.0, 0.0, 1.57, 0.0),
                              PoseObject(0.1, 0.1, 0.0, 0.0, 1.57, 0.0),
                              [0.1, -0.1, 0.0, 0.0, 1.57, 0.0]]

        unit_test_ws_points = [ws_name,
                               [0.3, -0.1, 0.0],
                               [0.3, 0.1, 0.0],
                               [0.2, 0.1, 0.0],
                               [0.2, -0.1, 0.0]]

        self.assertIsNone(self.vision.save_workspace_from_robot_poses(*unit_test_ws_poses))
        self.assertTrue(ws_name in self.vision.get_workspace_list())
        self.assertAlmostEquals(self.vision.get_workspace_ratio(ws_name), 1.0, places=2)
        # with self.assertRaises(RobotCommandException):
        #     self.vision.save_workspace_from_robot_poses(*unit_test_ws_poses)

        self.assertIsNone(self.vision.delete_workspace(ws_name))
        self.assertFalse(ws_name in self.vision.get_workspace_list())

        self.assertIsNone(self.vision.save_workspace_from_points(*unit_test_ws_points))
        self.assertTrue(ws_name in self.vision.get_workspace_list())
        self.assertAlmostEquals(self.vision.get_workspace_ratio(ws_name), 2.0, places=2)

        test_poses = self.vision.get_workspace_poses(ws_name)
        for pose_obtained, pose_expected in zip(test_poses, unit_test_ws_poses[0:]):
            self.assertAlmostEqualVector(pose_obtained, pose_expected.to_list()[:3], decimal=3)

        self.assertIsNone(self.vision.delete_workspace(ws_name))
        self.assertFalse(ws_name in self.vision.get_workspace_list())

    def test_target_from_rel(self):
        ws_name = "unit_test_ws"
        self.assertIsNone(self.vision.delete_workspace(ws_name))

        unit_test_ws_points = [ws_name,
                               [0.3, -0.1, 0.0],
                               [0.3, 0.1, 0.0],
                               [0.1, 0.1, 0.0],
                               [0.1, -0.1, 0.0]]

        self.assertIsNone(self.vision.save_workspace_from_points(*unit_test_ws_points))
        self.assertTrue(ws_name in self.vision.get_workspace_list())
        pose = self.vision.get_target_pose_from_rel(ws_name, 0.0, 0.5, 0.5, 0.0)
        self.assertIsInstance(pose, PoseObject)
        self.assertAlmostEqualVector(pose.to_list()[:3], [0.2, 0.0, 0.0])

        pose = self.vision.get_target_pose_from_rel(ws_name, 0.0, 0.0, 0.0, 0.0)
        self.assertIsInstance(pose, PoseObject)
        self.assertAlmostEqualVector(pose.to_list()[:3], unit_test_ws_points[1])

        pose = self.vision.get_target_pose_from_rel(ws_name, .10, 1.0, 1., 0.0)
        self.assertIsInstance(pose, PoseObject)
        expected_pose = unit_test_ws_points[-2][:]
        expected_pose[2] -= 0.1
        self.assertAlmostEqualVector(pose.to_list()[:3], expected_pose)

        self.assertIsNone(self.vision.delete_workspace(ws_name))


#
#
# @unittest.skipUnless(simulation, "Vision test is only coded for Gazebo")
# class TestVision(BaseTestTcpApi):
#     workspace_name = "gazebo_1"
#     workspace_h = 0.001
#     point_1 = [0.3369, 0.087, workspace_h]
#     point_2 = [point_1[0], -point_1[1], workspace_h]
#     point_3 = [0.163, -point_1[1], workspace_h]
#     point_4 = [point_3[0], point_1[1], workspace_h]
#
#     def setUp(self):
#         super(TestVision, self).setUp()
#         self.assertIsNone(self.niryo_robot.move_joints(0.0, 0.0, 0.0, 0.0, -1.57, 0.0))
#         self.assertIsNone(self.niryo_robot.update_tool())
#         self.assertIsNone(self.niryo_robot.save_workspace_from_points(
#             self.workspace_name, self.point_1, self.point_2, self.point_3, self.point_4))
#
#     def tearDown(self):
#         self.assertIsNone(self.niryo_robot.delete_workspace(self.workspace_name))
#         super(TestVision, self).tearDown()
#
#     def test_vision_detect(self):
#         # Getting img compressed & calibration object
#         self.assertIsNotNone(self.niryo_robot.get_img_compressed())
#         self.assertIsNotNone(self.niryo_robot.get_camera_intrinsics())
#
#         # Getting target pose's from multiple ways
#         self.assertIsNotNone(self.niryo_robot.get_target_pose_from_rel(
#             self.workspace_name, 0.1, 0.5, 0.5, 0.0))
#
#         self.assertIsNotNone(self.niryo_robot.get_target_pose_from_cam(
#             self.workspace_name, 0.1, ObjectShape.ANY, ObjectColor.ANY))
#
#         self.assertIsNotNone(self.niryo_robot.detect_object(self.workspace_name, ObjectShape.ANY, ObjectColor.RED))
#
#     def test_vision_move(self):
#         # Test to move to the object
#         self.assertIsNotNone(self.niryo_robot.move_to_object(self.workspace_name, 0.1,
#                                                              ObjectShape.ANY, ObjectColor.GREEN))
#         # Going back to observation pose
#         self.assertIsNone(self.niryo_robot.move_joints(0.0, 0.0, 0.0, 0.0, -1.57, 0.0))
#         # Vision Pick
#         self.assertIsNotNone(self.niryo_robot.vision_pick(self.workspace_name, 0.1,
#                                                           ObjectShape.ANY, ObjectColor.BLUE))
#


def suite():
    suite = unittest.TestSuite()
    for function_name in test_order:
        suite.addTest(TestVision(function_name))
    return suite


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())



#!/usr/bin/env python
import unittest
import time
import random
from threading import Event

from pyniryo2.led_ring.enums import AnimationMode, LedMode
from pyniryo2.led_ring.led_ring import LedRing
from pyniryo2.exceptions import RobotCommandException
from pyniryo2.niryo_ros import NiryoRos

from pyniryo2.enums import RobotErrors

robot_ip_address = "127.0.0.1"
port = 9090

test_order = [
    "test_ledring_status",
    "test_solid",
    "test_set_led",
    "test_custom",
    "test_alternate",
    "test_flash",
    "test_chase",
    "test_go_up",
    "test_go_up_down",
    "test_breath",
    "test_snake",
    "test_rainbow",
    "test_rainbow_cycle",
    "test_rainbow_chase",
    "test_turn_off",
]

WHITE = [255.0, 255.0, 255.0]
GREEN = [0.0, 255.0, 0.0]
NONE = [0.0, 0.0, 0.0]
RED = [255.0, 0.0, 0.0]
YELLOW = [255.0, 255.0, 0.0]
BLUE = [0.0, 0.0, 255.0]


class BaseTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.client = NiryoRos(ip_address=robot_ip_address, port=port)
        cls.led_ring = LedRing(cls.client)

        print("-- Connected --")

    @classmethod
    def tearDownClass(cls):
        cls.led_ring.turn_off()
        cls.client.close()


# noinspection PyTypeChecker
class TestLedRing(BaseTest):
    nb_leds = 30

    def test_ledring_status(self):
        self.assertEqual(self.led_ring.get_status().mode, LedMode.USER)

    def test_flash(self):
        self.test_classic_animation(self.led_ring.flash, AnimationMode.FLASHING)

    def test_chase(self):
        self.test_classic_animation(self.led_ring.chase, AnimationMode.CHASE)

    def test_go_up(self):
        self.test_classic_animation(self.led_ring.go_up, AnimationMode.GO_UP)

    def test_go_up_down(self):
        self.test_classic_animation(self.led_ring.go_up_down, AnimationMode.GO_UP_AND_DOWN)

    def test_breath(self):
        self.test_classic_animation(self.led_ring.breath, AnimationMode.BREATH)

    def test_snake(self):
        self.test_classic_animation(self.led_ring.snake, AnimationMode.SNAKE)

    def test_rainbow(self):
        self.test_rainbow_animation(self.led_ring.rainbow, AnimationMode.RAINBOW)

    def test_rainbow_cycle(self):
        self.test_rainbow_animation(self.led_ring.rainbow_cycle, AnimationMode.RAINBOW_CYLE)

    def test_rainbow_chase(self):
        self.test_rainbow_animation(self.led_ring.rainbow_chase, AnimationMode.RAINBOW_CHASE)

    def test_turn_off(self):
        self.assertIsNone(self.led_ring.turn_off())

    def test_classic_animation(self, function, animation):
        color = [random.randint(0, 255) for _ in range(3)]
        duration = random.uniform(1, 3)
        iterations = random.randint(1, 10)
        period = duration / iterations

        start_time = time.time()
        self.assertIsNone(function(color, period=period, iterations=iterations, wait=True))
        self.assertTrue(0 <= time.time() - start_time - duration < 1)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        self.assertIsNone(function(color, period=period, iterations=iterations, wait=False))
        self.assertTrue(time.time() - start_time < 1)
        time.sleep(0.2)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, animation)
        self.assertEqual([status.r, status.g, status.b], color)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        end_event = Event()
        end_event.clear()

        def animation_callback(resp):
            elapsed_time = time.time() - start_time
            self.assertEqual(resp['status'], RobotErrors.SUCCESS.value)
            self.assertGreaterEqual(elapsed_time, duration - 1)
            self.assertLessEqual(elapsed_time, duration + 1)
            end_event.set()

        self.assertIsNone(function(color, period=period, iterations=iterations, wait=True, callback=animation_callback))
        self.assertTrue(end_event.wait(timeout=duration + 2))
        self.assertIsNone(self.led_ring.turn_off())

        with self.assertRaises(RobotCommandException):
            function(color, period=period, iterations=1.5, wait=True)

        with self.assertRaises(RobotCommandException):
            function(color, period=period, iterations=1, wait=50)

        with self.assertRaises(RobotCommandException):
            function([], period=period, iterations=1, wait=50)

        with self.assertRaises(RobotCommandException):
            function(10, period=period, iterations=1, wait=50)

    def test_rainbow_animation(self, function, animation):
        duration = random.uniform(2, 5)
        iterations = random.randint(1, 3)
        period = duration / iterations

        start_time = time.time()
        self.assertIsNone(function(period=period, iterations=iterations, wait=True))
        self.assertTrue(0 <= time.time() - start_time - duration < 1)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        self.assertIsNone(function(period=period, iterations=iterations, wait=False))
        self.assertTrue(time.time() - start_time < 1)
        time.sleep(0.2)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, animation)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        end_event = Event()
        end_event.clear()

        def animation_callback(resp):
            elapsed_time = time.time() - start_time
            self.assertEqual(resp['status'], RobotErrors.SUCCESS.value)
            self.assertGreaterEqual(elapsed_time, duration - 1)
            self.assertLessEqual(elapsed_time, duration + 1)
            end_event.set()

        self.assertIsNone(function(period=period, iterations=iterations, wait=True, callback=animation_callback))
        self.assertTrue(end_event.wait(timeout=duration + 2))
        self.assertIsNone(self.led_ring.turn_off())

        with self.assertRaises(RobotCommandException):
            function(period=period, iterations=1.5, wait=True)

        with self.assertRaises(RobotCommandException):
            function(period=period, iterations=1, wait=50)

    def test_alternate(self):
        colors = [[random.randint(0, 255) for _ in range(3)] for _ in range(3)]
        duration = random.uniform(1, 3)
        iterations = random.randint(1, 10)
        period = duration / iterations

        start_time = time.time()
        self.assertIsNone(self.led_ring.alternate(colors, period=period, iterations=iterations, wait=True))
        self.assertTrue(0 <= time.time() - start_time - duration < 1)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        self.assertIsNone(self.led_ring.alternate(colors, period=period, iterations=iterations, wait=False))
        self.assertTrue(time.time() - start_time < 1)
        time.sleep(0.2)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, AnimationMode.ALTERNATE)
        self.assertTrue([status.r, status.g, status.b] in colors)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        end_event = Event()
        end_event.clear()

        def animation_callback(resp):
            elapsed_time = time.time() - start_time
            self.assertEqual(resp['status'], RobotErrors.SUCCESS.value)
            self.assertGreaterEqual(elapsed_time, duration - 1)
            self.assertLessEqual(elapsed_time, duration + 1)
            end_event.set()

        self.assertIsNone(self.led_ring.alternate(colors, period=period, iterations=iterations, wait=True,
                                                  callback=animation_callback))
        self.assertTrue(end_event.wait(timeout=duration + 2))

        def animation_callback_stop(resp):
            self.assertEqual(resp['status'], RobotErrors.STOPPED.value)
            self.assertGreater(time.time() - start_time, 1)

        self.assertIsNone(self.led_ring.alternate(colors, period=period, iterations=iterations, wait=True,
                                                  callback=animation_callback_stop))
        self.assertIsNone(self.led_ring.turn_off())

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(colors, period=period, iterations=1.5, wait=True)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(colors, period=period, iterations=1, wait=50)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate([], period=period, iterations=1, wait=50)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(10, period=period, iterations=1, wait=50)

    def test_wipe(self):
        color = [random.randint(0, 255) for _ in range(3)]
        period = random.uniform(1, 4)

        start_time = time.time()
        self.assertIsNone(self.led_ring.wipe(color, period=period, wait=True))
        self.assertTrue(0 <= time.time() - start_time - period < 1)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        self.assertIsNone(self.led_ring.alternate(color, period=period, wait=False))
        self.assertTrue(time.time() - start_time < 1)
        time.sleep(0.2)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, AnimationMode.COLOR_WIPE)
        self.assertTrue([status.r, status.g, status.b] == color)
        self.assertIsNone(self.led_ring.turn_off())

        start_time = time.time()
        end_event = Event()
        end_event.clear()

        def animation_callback(resp):
            elapsed_time = time.time() - start_time
            self.assertEqual(resp['status'], RobotErrors.SUCCESS.value)
            self.assertGreaterEqual(elapsed_time, period - 1)
            self.assertLessEqual(elapsed_time, period + 1)
            end_event.set()

        self.assertIsNone(self.led_ring.alternate(color, period=period, wait=True, callback=animation_callback))
        self.assertTrue(end_event.wait(timeout=period + 2))
        self.assertIsNone(self.led_ring.turn_off())

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(color, period=[], wait=True)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(color, period=period, wait=50)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate([], period=period, iterations=1, wait=50)

        with self.assertRaises(RobotCommandException):
            self.led_ring.alternate(10, period=period, iterations=1, wait=50)

    def test_set_led(self):
        for led_id in range(30):
            self.assertIsNone(self.led_ring.set_led_color(led_id, [0, 255, 255]))

    def test_solid(self):
        color = [255, 0, 255]
        self.assertIsNone(self.led_ring.solid(color))
        time.sleep(0.5)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, AnimationMode.SOLID)
        self.assertEqual([status.r, status.g, status.b], [255, 0, 255])

        with self.assertRaises(RobotCommandException):
            self.led_ring.solid([])

        with self.assertRaises(RobotCommandException):
            self.led_ring.solid(10)

    def test_custom(self):
        self.assertIsNone(self.led_ring.custom(led_colors=[[i / 30. * 255, 0, 255 - i / 30.] for i in range(30)]))
        time.sleep(1)
        status = self.led_ring.get_status()
        self.assertEqual(status.mode, LedMode.USER)
        self.assertEqual(status.animation, AnimationMode.CUSTOM)

        with self.assertRaises(RobotCommandException):
            self.led_ring.custom(led_colors=[20 * [3 * [0]]])

        with self.assertRaises(RobotCommandException):
            self.led_ring.custom(led_colors=[30 * [2 * [0]]])


def suite():
    suite = unittest.TestSuite()
    for function_name in test_order:
        suite.addTest(TestLedRing(function_name))
    return suite


if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite())



# Communication imports
from pyniryo2.robot_commander import RobotCommander

from .enums import ConveyorID, ConveyorCan, ConveyorTTL, ConveyorDirection, ConveyorStatus
from .services import ConveyorServices
from .topics import ConveyorTopics, conveyor_id_to_conveyor_type


class Conveyor(RobotCommander):
    # --- Public functions --- #
    def __init__(self, client):
        """
        Conveyor robot functions

        Example: ::

            ros_instance = NiryoRos("10.10.10.10") # Hotspot
            conveyor_interface = Conveyor(ros_instance)

        :param client: Niryo ROS client
        :type client: NiryoRos
        """
        super(Conveyor, self).__init__(client)

        self._services = ConveyorServices(self._client)
        self._topics = ConveyorTopics(self._client)

    def set_conveyor(self):
        """
        Scan if a conveyor is plugged or not on a can bus. 
        If a new conveyor is detected, activate it and return its conveyor ID. 
        If a conveyor is already set, return its ID

        Example: ::

            # Get the id of the conveyor plugged
            conveyor_id = conveyor.set_conveyor()

            # Scan and set the conveyor plugged
            conveyor.set_conveyor()

        :return : New conveyor ID
        :rtype: ConveyorID
        """
        req = self._services.get_ping_and_set_conveyor_request()
        resp = self._services.ping_and_set_conveyor_service.call(req)
        conveyor_id = conveyor_id_to_conveyor_type(resp["id"])

        # If new conveyor has been found
        if conveyor_id != ConveyorID.NONE:
            # print("New conveyor detected and set with id :", conveyor_id)
            return conveyor_id
        else:
            last_conveyor_id = self.get_conveyors_feedback()[0].conveyor_id
            if last_conveyor_id != ConveyorID.NONE:
                # print("No new conveyor detected, actual conveyor id :", last_conveyor_id)
                return last_conveyor_id
            else:
                # print("No conveyor detected")
                return last_conveyor_id

    def unset_conveyor(self, conveyor_id):
        """
        Remove and unset a conveyor previously plugged and set

        Example: ::

            conveyor_id = conveyor.set_conveyor()
            conveyor.unset_conveyor(conveyor_id)
            conveyor.unset_conveyor(ConveyorID.ID_1)
            conveyor.unset_conveyor(ConveyorID.ID_2)
        
        :param conveyor_id: Basically, ConveyorID.ID_1 or ConveyorID.ID_2
        :type conveyor_id: ConveyorID
        :rtype: None
        """
        self._check_instance(conveyor_id, (ConveyorID, ConveyorTTL, ConveyorCan))
        real_conv_id = self.__conveyor_number_to_conveyor_id(conveyor_id)

        req = self._services.unset_conveyor_request(real_conv_id)
        resp = self._services.ping_and_set_conveyor_service.call(req)
        self._check_result_status(resp)

    def run_conveyor(self, conveyor_id, speed=100, direction=ConveyorDirection.FORWARD):
        """
        Run conveyor at id 'conveyor_id'

        Example: ::

            # Set the conveyor and get its id and un it. 
            # By default, the conveyor will go forward at a speed of 50
            # You can't choose the parameters with this method

            conveyor_id = conveyor.set_conveyor()
            conveyor.run_conveyor(conveyor_id) 

        :param conveyor_id: The conveyor id
        :type conveyor_id: ConveyorID
        :param speed: speed percentage between 0% and 100%
        :type speed: int
        :param direction: direction = ConveyorDirection.FORWARD
        :type direction: ConveyorDirection
        :rtype: None
        """
        self.control_conveyor(conveyor_id, control_on=True, speed=speed, direction=direction)

    def stop_conveyor(self, conveyor_id):
        """
        Stop conveyor at id 'conveyor_id'
    
        Example: ::

            # Set the conveyor and get its id, run it and then stop it after 3 seconds
            # By default, the conveyor will go forward at a speed of 50
            # When the conveyor is stopped, its control_on parameter is False and its speed is 0

            import time

            conveyor_id = conveyor.set_conveyor()
            conveyor.run_conveyor(conveyor_id)
            time.sleep(3)
            conveyor.stop_conveyor(conveyor_id) 

        :param conveyor_id:
        :type conveyor_id: ConveyorID
        :rtype: None
        """
        self.control_conveyor(conveyor_id, control_on=False, speed=50, direction=ConveyorDirection.FORWARD)

    def control_conveyor(self, conveyor_id, control_on, speed, direction):
        """
        Control conveyor associated to conveyor_id.
        Then stops it if bool_control_on is False, else refreshes it speed and direction

        Example: ::

            # Example 1
            # Set the conveyor and get its id, control it and then stop it after 3 seconds
            # It this first example, we control the conveyor at a speed of 100% and in the forward direction

            import time

            conveyor_id = conveyor.set_conveyor()
            conveyor.control_conveyo(conveyor_id, True, 100, ConveyorDirection.FORWARD.value)
            time.sleep(3)
            conveyor.stop_conveyor(conveyor_id) 

        # Example 2
            # Set the conveyor and get its id, control it and then stop it after 3 seconds
            # It this second example, we control the conveyor at a speed of 30% and in the backward direction

            import time

            conveyor_id = conveyor.set_conveyor()
            conveyor.control_conveyor(conveyor_id, True, 30, ConveyorDirection.BACKWARD.value)
            time.sleep(3)
            conveyor.stop_conveyor(conveyor_id) 

        :param conveyor_id:
        :type conveyor_id: ConveyorID
        :param control_on: True for activate, False for deactivate
        :type control_on: bool
        :param speed: New speed which is a percentage of maximum speed (0% to 100%)
        :type speed: int
        :param direction: ConveyorDirection.FORWARD.value, ConveyorDirection.BACKWARD.value
        :type direction: ConveyorDirection
        :rtype: None
        """
        self._check_instance(conveyor_id, (ConveyorID, ConveyorTTL, ConveyorCan))
        self._check_type(control_on, bool)
        self._transform_to_type(speed, int)
        self._check_range_belonging(speed, 0, 100)
        self._check_enum_belonging(direction, ConveyorDirection)

        real_conv_id = self.__conveyor_number_to_conveyor_id(conveyor_id)
        req = self._services.control_conveyor_request(real_conv_id, control_on, speed, direction)
        resp = self._services.control_conveyor_service.call(req)

        self._check_result_status(resp)

    @property
    def get_conveyors_feedback(self):
        """
        Returns the conveyors feedback client which can be used synchronously or asynchronously
        to obtain the conveyors feedback: (conveyor_id, connection_state, running, speed, direction)

        Examples: ::

            # Get last value
            arm.get_conveyors_feedback()
            arm.get_conveyors_feedback.value

            # Subscribe a callback
            def conveyor_callback(conveyor_feedback):
                print conveyor_feedback

            arm.hardware_status.subscribe(conveyor_callback)
            # wait
            arm.hardware_status.unsubscribe()

        :return: namedtuple[conveyor_id, running, speed, direction]
        :rtype: namedtuple(ConveyorID, bool, int, ConveyorDirection)
        """
        return self._topics.conveyor_feedback_topic

    @property
    def conveyors(self):
        """
        Return list of registered conveyors

        :return: namedtuple[conveyor_id, running, speed, direction]
        :rtype: namedtuple(ConveyorID, bool, int, ConveyorDirection)
        """
        return self._topics.conveyor_feedback_topic()

    def __conveyor_number_to_conveyor_id(self, conveyor_number):
        if conveyor_number == ConveyorID.ID_1:
            return ConveyorTTL.ID_1 if self._client.hardware_version in ['ned2'] else ConveyorCan.ID_1
        elif conveyor_number == ConveyorID.ID_2:
            return ConveyorTTL.ID_2 if self._client.hardware_version in ['ned2'] else ConveyorCan.ID_2
        else:
            return conveyor_number

    @staticmethod
    def __conveyor_id_to_conveyor_number(conveyor_id):
        if conveyor_id in [ConveyorTTL.ID_1, ConveyorCan.ID_1]:
            return ConveyorID.ID_1
        elif conveyor_id in [ConveyorTTL.ID_2, ConveyorCan.ID_2]:
            return ConveyorID.ID_2
        elif conveyor_id in [ConveyorTTL.NONE, ConveyorCan.NONE]:
            return ConveyorID.NONE
        else:
            return conveyor_id



import roslibpy

from pyniryo2.objects import PoseObject
from pyniryo2.utils import pose_dict_to_list

from pyniryo2.saved_poses.enums import ManagePose


class SavedPosesServices(object):

    def __init__(self, client):
        self.__client = client

        self.manage_pose_service = roslibpy.Service(self.__client,
                                                    '/niryo_robot_poses_handlers/manage_pose',
                                                    'niryo_robot_poses_handlers/ManagePose')

        self.get_pose_list_service = roslibpy.Service(self.__client,
                                                      '/niryo_robot_poses_handlers/get_pose_list',
                                                      'niryo_robot_poses_handlers/GetNameDescriptionList')

        self.get_pose_service = roslibpy.Service(self.__client,
                                                 '/niryo_robot_poses_handlers/get_pose',
                                                 'niryo_robot_poses_handlers/GetPose')

    @staticmethod
    def save_joint_pose_request(name, joints, description=""):
        niryo_pose = roslibpy.ServiceRequest({"name": name, "description": description, "joints": joints})
        return roslibpy.ServiceRequest({"cmd": ManagePose.SAVE.value, "pose": niryo_pose})

    @staticmethod
    def save_pose_request(name, pose, description=""):
        niryo_pose = {"name": name, "description": description, "position": dict(zip(["x", "y", "z"], pose[:3]))}

        if len(pose) == 7:
            niryo_pose["orientation"] = dict(zip(["x", "y", "z", "w"], pose[3:]))
        else:
            niryo_pose["rpy"] = dict(zip(["roll", "pitch", "yaw"], pose[3:]))

        return roslibpy.ServiceRequest({"cmd": ManagePose.SAVE.value, "pose": niryo_pose})

    @staticmethod
    def delete_pose_request(name):
        return roslibpy.ServiceRequest({"cmd": ManagePose.DELETE.value, "pose": {"name": name}})

    @staticmethod
    def get_pose_list_request():
        return roslibpy.ServiceRequest()

    @staticmethod
    def get_saved_pose_request(name):
        return roslibpy.ServiceRequest({"name": name})

    @staticmethod
    def get_saved_pose_response_to_pose(response):
        return PoseObject(*pose_dict_to_list(response["pose"]))

    @staticmethod
    def get_saved_pose_list_response_to_list(response):
        return [str(pose_name) for pose_name in response["name_list"]]



import roslibpy.actionlib

from pyniryo2.enums import ArmMoveCommandType

class ArmActions(object):

    def __init__(self, client):
        self.__client = client

        self.arm_action = None
        self.arm_action = roslibpy.actionlib.ActionClient(self.__client,
                                                      '/niryo_robot_arm_commander/robot_action/',
                                                      'niryo_robot_arm_commander/RobotMoveAction')

    def __del__(self):
        if self.arm_action is not None:
            self.arm_action.cancel()
            self.arm_action.dispose()

    def get_move_joints_goal(self, joints):
        cmd = {'cmd_type': ArmMoveCommandType.JOINTS.value, 'joints': joints}
        return roslibpy.actionlib.Goal(self.arm_action, roslibpy.Message({'cmd': cmd}))

    def get_move_pose_goal(self, pose_list):
        cmd = self.pose_list_to_dict(pose_list)
        cmd['cmd_type'] = ArmMoveCommandType.POSE.value
        return roslibpy.actionlib.Goal(self.arm_action, roslibpy.Message({'cmd': cmd}))

    def get_move_linear_pose_goal(self, pose_list):
        cmd = self.pose_list_to_dict(pose_list)
        cmd['cmd_type'] = ArmMoveCommandType.LINEAR_POSE.value
        return roslibpy.actionlib.Goal(self.arm_action, roslibpy.Message({'cmd': cmd}))

    def get_shift_pose_goal(self, axis, shift_value):
        shift_cmd = {'axis_number': axis, 'value': shift_value}
        cmd = {'cmd_type': ArmMoveCommandType.SHIFT_POSE.value, 'shift': shift_cmd}
        return roslibpy.actionlib.Goal(self.arm_action, roslibpy.Message({'cmd': cmd}))

    @staticmethod
    def pose_list_to_dict(pose_list):
        return {"position": {"x": pose_list[0], "y": pose_list[1], "z": pose_list[2]},
                "rpy": {"roll": pose_list[3], "pitch": pose_list[4], "yaw": pose_list[5]}}
